# New BSD License
# ===============
# Copyright (c) 2007, Graham Binns http://launchpad.net/~gmb
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#
#     * Neither the name of the <ORGANIZATION> nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Actions module for the clog application."""

__metaclass__ = type

__all__ = [
    'Action',
    'get_action',
    'NotEnoughArgumentsError',
    'UnknownActionError']

from ConfigParser import ConfigParser
from optparse import make_option, OptionParser

from cloglib import config
from cloglib.blogpost import BlogPost
from cloglib.metaweblog import MetaWeblog


post_template = """
Subject:
Categories:

[The body of your post]
"""
registered_actions = {}


class NoSuchSiteError(Exception):
    """Raised when a site passed to an XMLRPCAction doesn't exist."""

class NotEnoughArgumentsError(Exception):
    """Raised when not enough arguments are passed to a command."""


class UnknownActionError(Exception):
    """Raised when an unknown action is requested."""


def get_action(action, arguments):
    """Return the callable action for a given action argument."""
    try:
        action_cls = registered_actions[action]
        action = action_cls(arguments)
        action.initialise()
        return action
    except KeyError:
        raise UnknownActionError(action)


def get_action_list():
    """Return a list of actions as (name, summary) tuples."""
    return registered_actions.items()


def register_action(name, action):
    """Register an action."""
    # action should subclass action.
    assert issubclass(action, Action), (
        "Action must subclass cloglib.actions.Action.")

    if name not in registered_actions:
        registered_actions[name] = action


class Action:
    """A base Action class that can be subclassed to create clog actions."""

    name = "base"
    summary = "There is no help available for the action %s."
    usage = "There is no usage description for this action."
    min_args = 0
    config_file = None

    def __init__(self, arguments):
        """Create a new Action with the passed arguments."""
        # We parse any config file first so that command line options
        # take precedent over it.
        self.config = ConfigParser()
        if self.config_file is not None:
            self.config.read([config.CONFIG_DIR + '/' + self.config_file])

        self.option_parser = OptionParser(option_list=self.getOptions())
        (self.options, self.args) = self.option_parser.parse_args(arguments)
        self.option_parser.usage = self.usage

        # We print the help string if there aren't any arguments.
        if len(self.args) < self.min_args:
            option_parser.print_help()
            raise NotEnoughArgumentsError()

    def initialise(self):
        """Do any setup required for this action.

        Action subclasses should do their setup here rather than in
        __init__() so that they can be instantiated without the setup
        having to be carried out (for example by the Help action.)
        """
        pass

    def getOptions(self):
        """Return a list of option parser options for this action."""
        return [
            make_option('-c', '--use-config', dest='config_dir', help='')]


class Help(Action):
    """An action to print the help text for any other action."""

    name = "help"
    summary = "Display the help for an action."
    usage = "%prog help foo: print all the help for the action 'foo'."

    def formatActionList(self, actions):
        """Return a list of actions formatted as a string."""
        action_base_string = "  clog %(name)s: %(summary)s\n"

        action_list = "Available actions:\n\n"

        for name, action in actions:
            action_string = action_base_string % {
                'name': name, 'summary': action.summary}

            action_list = action_list + action_string

        return action_list

    def run(self):
        """Display the help for a specified action."""
        # If there are no arguments we simply print the list of
        # registered commands.
        if len(self.args) == 0 or self.args[0] == 'commands':
            print self.formatActionList(get_action_list())
            return

        # The first argument will be the action that we want to show the
        # help for.
        if self.args[0] in registered_actions:
            action_cls = registered_actions[self.args[0]]
            action = action_cls([])
            print action.summary
            action.option_parser.print_help()

register_action('help', Help)

